import type Player from '@oplayer/core'
import { loading } from '../style'
import { isSafari } from '@oplayer/core'

const loadingListener = (player: Player) => {
  const addClass = () => player.$root.classList.add(loading)
  const removeClass = () => {
    if (!player.isSourceChanging) {
      player.$root.classList.remove(loading)
    }
  }

  if (player.$video.preload != 'none') {
    addClass()
  }

  player.on('loadstart', () => {
    if (player.$video.preload == 'none') removeClass()
  })

  player.on(['loading', 'seeking', 'videoqualitychange', 'videosourcechange', 'waiting'], addClass)
  player.on(['loaded', 'canplaythrough', 'playing', 'pause', 'seeked', 'error'], removeClass)

  // safari 不预加载, 当 autoplay = true 才触发 canplay(预加载), 改变视频地址后默认预加载(或者是有用户交互?)
  // chrome 自动播放失败也可能无 canplay 事件
  player.on(player.options.autoplay || isSafari ? 'loadedmetadata' : 'canplay', () =>
    // 无视 isSourceChanging
    // 顺序: loadedmetadata -> ⬇(isSourceChanging: false) -> videosourcechanged(isSourceChanging: true)
    player.$root.classList.remove(loading)
  )

  let prevPlayStateIsPaused: boolean = !player.isPlaying

  player.on(['play', 'pause'], () => {
    prevPlayStateIsPaused = !player.isPlaying
  })

  // why playing:  暂停很久后在播放可能会卡很久
  player.on('playing', () => {
    if (prevPlayStateIsPaused) {
      const timeWhenWaiting = player.currentTime
      // Browsers may emit a timeupdate event after a waiting event. In order to prevent
      // premature removal of the waiting class, wait for the time to change.
      const timeUpdateListener = () => {
        if (timeWhenWaiting !== player.currentTime) {
          removeClass()
          player.off('timeupdate', timeUpdateListener)
        } else {
          addClass()
        }
      }

      player.on('timeupdate', timeUpdateListener)
    }
  })
}

const isLoading = (player: Player) => player.$root.classList.contains(loading)

export { isLoading, loadingListener }
